#!/usr/bin/python

##############################################################################
# opensslcerts2ldif.py 0.1.1
#
# Alle gueltigen Zertifikate einer Zertifizierungsstelle 
# in eine LDIF-Datei auf stdout schreiben.
##############################################################################

##############################################################################
# Hier einige Variablen zur Konfiguration
##############################################################################

import openssl

# Umbenennungstabelle SSLeay-Attributname -> LDAP-Attributname
# Mu angepat werden, falls in SSLeay zustzliche OIDs verwendet werden
ldifattrname = {
                 'C':'c',
                 'ST':'st',
                 'L':'l',
                 'O':'o',
                 'OU':'ou',
                 'CN':'cn',
                 'I':'initials',
                 'uid':'uid',
                 'Email':'mail'
               }

dn_template = 'cn=%s,mail=%s,ou=%s,o=%s,c=%s'

ldif_entryobjectclass = 'person'

# Pfadname der SSLeay-Datenbank
openssl.db_filename = '/usr/local/ssl/TestCA/index.txt'
# Verzeichnisname mit SSLeay-Zertfikaten im PEM-Format
openssl_certdir    = '/usr/local/ssl/TestCA/newcerts'
# Pfadname des SSLeay-Programms
openssl_bin         = '/usr/local/ssl/bin/openssl'
# Pfadname des SSLeay-Programms
ldif_bin         = '/usr/local/bin/ldif'

##############################################################################
# Ab hier gibt es nix mehr zu konfigurieren
##############################################################################

import sys, os, string, charset, getopt, pipes

ldifattrnamekeys=ldifattrname.keys()
opensslattrname = {}
for i in ldifattrnamekeys:
  opensslattrname[ldifattrname[i]]=i
opensslattrnamekeys=opensslattrname.keys()

def findoption(options,paramname):
  for i in options:
    if i[0]==paramname:
      return i
  return ()

def PrintErrorMsg(ErrorMsg='',ErrorCode=1,NoExit=0):
  sys.stderr.write('*** Error: %s\n\n' % ErrorMsg)
  if NoExit:
    return
  else:
    sys.exit(ErrorCode)

def PrintUsage(ErrorMsg='',ErrorCode=1):
  script_name = string.split(sys.argv[0],os.sep)[-1]
  sys.stderr.write("""*** %s *** (C) by Michael Stroeder, 1998

usage: %s [-o objectclass] [-d database] [-c certdir] [-s openssl]
	  [filter 1] ... [filter n]

Write all SSLeay-entries matching all filters to a LDIF file on stdout.
A filter has the form <attribute>=<value rexexp.> .

Options:

       -o objectclass
              Specify an objectclass which is used for the entries.
	      Default: %s

       -d database
              Specify the pathname of the SSLeay database file.
	      Default: %s

       -c certdir
              Specifies the directory where the certificates are located.
	      Default: %s

       -s openssl
              Specify the pathname of the SSLeay binary.
	      Default: %s

       -l ldif
              Specify the pathname of the ldif binary.
	      Default: %s

""" % (script_name,script_name,ldif_entryobjectclass,\
       openssl.db_filename,openssl_certdir,openssl_bin,ldif_bin))
  if ErrorMsg:
    sys.stderr.write('*** Error: %s\n\n' % ErrorMsg)
  sys.exit(ErrorCode)

##############################################################################
# __main__()
##############################################################################

# Kommandozeile parsen
try:
  options,args=getopt.getopt(sys.argv[1:],'o:d:c:s:',[])
except getopt.error:
  PrintUsage('Wrong options!')

# objectclass
try:
  dummy,ldif_entryobjectclass = findoption(options,'-o')
except: pass

# Pfadname der SSLeay-Datenbank
try:
  dummy,openssl.db_filename = findoption(options,'-d')
except: pass
if not os.path.isfile(openssl.db_filename):
  PrintUsage('SSLeay database "%s" not found.' % openssl.db_filename)

# Verzeichnisname mit SSLeay-Zertfikaten im PEM-Format
try:
  dummy,openssl_certdir = findoption(options,'-c')
except: pass
if not os.path.isdir(openssl_certdir):
  PrintUsage('Directory "%s" with certificates not found.' % openssl_certdir)

# Pfadname des SSLeay-Programms
try:
  dummy,openssl_bin = findoption(options,'-s')
except: pass
if not os.path.isfile(openssl_bin):
  PrintUsage('SSLeay binary "%s" not found.' % openssl_bin)

if args:
  filter = {}
  for filterarg in args:
    try:
      filterattr,filtervalue = string.split(filterarg,'=')
    except ValueError:
      PrintUsage('Wrong filter format!')
    if filterattr in ldifattrnamekeys:
      filter[string.strip(filterattr)]=string.strip(filtervalue)
    else:
      PrintUsage('Wrong filter attribute name!')
else:
  PrintUsage('No filter given.')

allcertusers = openssl.GetEntriesbyDN(filter,0,1)

if not allcertusers:
  sys.stderr.write('No matching entries found in SSLeay database.\n')
  sys.exit(99)
  
sys.stderr.write('Found %d matching and valid entries in SSLeay database.\n' % len(allcertusers))

# Eintraege schreiben
for certuser in allcertusers:

  openssldn = openssl.SplitDN(charset.iso2utf(charset.trans(certuser[openssl.DB_name],charset.t61,charset.iso)))

  pemfilename = os.path.join(openssl_certdir,certuser[openssl.DB_serial]+'.pem')
  if not os.path.isfile(pemfilename):
    sys.stderr.write('Certificate %s file not found!\n' % pemfilename)

  # DN aus Template zusammenbauen
  dntemplatelist = string.split(dn_template,',')
  dnlist = []
  for i in dntemplatelist:
    attr,format = string.split(i,'=',1)
    if openssldn.has_key(opensslattrname[attr]) and openssldn[opensslattrname[attr]]:
      dnlist.append(i % (openssldn[opensslattrname[attr]]))
  dn = string.join(dnlist,',')

  sys.stdout.write('dn:%s\n' % (dn))
  sys.stdout.write('objectclass: top\nobjectclass: strongauthenticationuser\n')
  sys.stdout.write('objectclass: %s\n' % ldif_entryobjectclass)
  for i in opensslattrnamekeys:
    try:
      sys.stdout.write('%s: %s\n' % (i,openssldn[opensslattrname[i]]))
    except KeyError: pass

  # Pfeife ausfhren

  # openssl x509 eine DER-Datei auf stdout schreiben lassen
  pipe = os.popen('%s x509 -inform pem -outform der -in %s | %s -b "usercertificate;binary"' % (openssl_bin,pemfilename,ldif_bin))
  s=pipe.readline()
  while s:
    sys.stdout.write(s)
    s=pipe.readline()

  # Leerzeile zum Trennen der Eintraege
  sys.stdout.write('\n')
