#!/usr/bin/python

########################################################################
# print-ca-certs.py
# (c) by Michael Stroeder, michael.stroeder@inka.de
########################################################################

Version = '0.5.3'

########################################################################
# This simple script prints all CA certs on stdout.
# This is intended to generate a authentic printout of the fingerprints
# on the private CA system.
# Choose the option --html to generate nicer formatted HTML-output
# instead of the default textual output in ISO-8859-1.
########################################################################


import sys, string, os, getopt, pycacnf, openssl, charset, htmlbase

from pycacnf import opensslcnf, pyca_section

def findoption(options,paramname):
  for i in options:
    if i[0]==paramname:
      return i
  return ()

def PrintUsage(ErrorMsg='',ErrorCode=1):
  script_name = string.split(sys.argv[0],os.sep)[-1]
  sys.stderr.write("""*** %s *** (C) by Michael Stroeder, 1999

usage: %s [options]

Options:

  -h or --help
         Print out this message

  --html
         Generate nicer formatted HTML output

""" % (script_name,script_name))
  sys.exit(ErrorCode)

ca_names = opensslcnf.sectionkeys.get('ca',[])

script_name=sys.argv[0]

try:
  options,args=getopt.getopt(sys.argv[1:],'h',['help','html'])
except getopt.error:
  PrintUsage(script_name)

sys.stderr.write('***options=%s\nargs%s\n' %  (options,args))

if ('h' in options) or ('--help' in options):
  PrintUsage(script_name)

htmlmode = findoption(options,'--html')!=()

if htmlmode:

  #HTML mode

  sys.stdout.write('<HTML><HEAD><TITLE>CA certs</TITLE></HEAD><BODY><CENTER>')

  for ca_name in ca_names:

    ca = opensslcnf.getcadata(ca_name)

    if os.path.isfile(ca.certificate):

      # Parse certificate textual output
      cacert = openssl.cert.X509CertificateClass(ca.certificate)
      # Convert character sets
      attrlist = ['CN','Email','OU','O','L','ST','C']
      subjectdatalist = []
      for attr in attrlist:
        subjectdatalist.append(string.strip(charset.trans(cacert.subject.get(attr,''),charset.t61,charset.html)))
      issuerdatalist = []
      for attr in attrlist:
        issuerdatalist.append(string.strip(charset.trans(cacert.issuer.get(attr,''),charset.t61,charset.html)))

      sys.stdout.write("""<H2>%s</H2>
<TABLE ALIGN=MIDDLE BORDER=1 WIDTH="80%%">
<TR><TH WIDTH="50%%">Subject</TH><TH WIDTH="50%%">Issuer</TH></TR>
<TR><TD WIDTH="50%%">%s</TD><TD WIDTH="50%%">%s</TD></TR>
</TABLE>
<TABLE ALIGN=MIDDLE BORDER=1 WIDTH="80%%">
<TR><TD WIDTH="15%%">Serial:</TD><TD>%s</TD></TR>
<TR><TD WIDTH="15%%">Validity:</TD><TD>from %s until %s</TD></TR>
<TR><TD WIDTH="15%%">%s Fingerprint:</TD><TD>%s</TD></TR>
</TABLE><P>
""" % ( \
        ca_name,
        string.join(subjectdatalist,'<BR>'), \
        string.join(issuerdatalist,'<BR>'), \
	cacert.serial,cacert.notBefore,cacert.notAfter,cacert.digest,cacert.fingerprint
       ))

  sys.stdout.write('</CENTER><BODY>\n<HTML>\n')
  

else:

  # Text mode

  for ca_name in ca_names:

    ca = opensslcnf.getcadata(ca_name)

    if os.path.isfile(ca.certificate):

      # Parse certificate textual output
      cacert = openssl.cert.X509CertificateClass(ca.certificate)

      # Convert character sets
      for dict in [cacert.issuer,cacert.subject]:
	for attr in dict.keys():
          dict[attr] = charset.trans(dict[attr],charset.t61,charset.iso)

      sys.stdout.write('Subject:\nCommon Name: "%(CN)s"\nOrganizational Unit: "%(OU)s"\nOrganization: "%(O)s"\nLocation: "%(L)s"\nState/Province: "%(ST)s"\nCountry: "%(C)s"\n\n' % (cacert.subject))
      sys.stdout.write('Issuer:\nCommon Name: "%(CN)s"\nOrganizational Unit: "%(OU)s"\nOrganization: "%(O)s"\nLocation: "%(L)s"\nState/Province: "%(ST)s"\nCountry: "%(C)s"\n' % (cacert.issuer))
      sys.stdout.write('Serial: %s\n' % (cacert.serial))
      sys.stdout.write('Validity: from %s until %s\n' % (cacert.notBefore,cacert.notAfter))
      sys.stdout.write('%s Fingerprint: %s\n\n%s\n\n' % (cacert.digest,cacert.fingerprint,72*'-'))
